/* (C) 2022 YiRing, Inc. */
package com.yiring.app.design.strategy.rule;

import com.yiring.app.constant.alarm.RelevanceParamEnum;
import com.yiring.app.design.strategy.IParamInitStrategy;
import com.yiring.app.design.strategy.IScanAlarmStrategy;
import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.domain.location.*;
import com.yiring.app.param.location.fence.LocationFenceJobParam;
import com.yiring.app.param.location.rule.*;
import com.yiring.app.util.TimeUtil;
import com.yiring.app.vo.location.fence.LocationFenceJobVo;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

/**
 * 不允许离开人员相关策略
 * @author tml
 * @version 1.0
 * @date 2022/4/29 9:32
 */
@Component
public class NotAllowLeaveStrategy implements IParamInitStrategy<LocationFenceRuleAddParam>, IScanAlarmStrategy {

    @Resource
    private LocationTurnoverRepository locationTurnoverRepository;

    @Resource
    private UserRepository userRepository;

    public NotAllowLeaveStrategy() {
        Integer relevanceParam = RelevanceParamEnum.NOT_ALLOW_LEAVE.getCode();
        LocationFenceRuleContext.register(relevanceParam, this);
        ScanAlarmContext.register(relevanceParam, this);
    }

    @Override
    public Result<String> paramInit(LocationFenceRuleAddParam param) {
        RuleParam ruleParam = param.getRuleParam();
        List<TimeAndUserParam> timeAndUser = ruleParam.getTimeAndUser();
        if (CollectionUtils.isEmpty(timeAndUser)) {
            return Result.no(Status.EXPECTATION_FAILED, "请至少配置一条不允许离开人员的规则");
        }
        for (TimeAndUserParam item : timeAndUser) {
            if (CollectionUtils.isEmpty(item.getUser())) {
                return Result.no(Status.EXPECTATION_FAILED, "请至少配置一个不允许离开的人员");
            }
        }
        //如果只设置星期没有设置时间，则默认时间是全天
        for (TimeAndUserParam item : timeAndUser) {
            for (TimeParam time : item.getTime()) {
                if (time.getWeek() != null) {
                    if (time.getBeginTime() == null) {
                        time.setBeginTime("00:00:00");
                    }
                    if (time.getEndTime() == null) {
                        time.setEndTime("23:59:59");
                    }
                }
            }
        }
        //检查配置的员工是否存在
        Specification<User> specification = (root, query, criteriaBuilder) -> {
            CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get(User.Fields.uuid));
            HashSet<String> ids = new HashSet<>();
            for (TimeAndUserParam item : timeAndUser) {
                for (RelevanceUserParam userParam : item.getUser()) {
                    ids.add(userParam.getUuid());
                }
            }
            for (String uuid : ids) {
                in.value(uuid);
            }
            return criteriaBuilder.and(in);
        };
        Map<String, List<User>> map = userRepository
            .findAll(specification)
            .stream()
            .collect(Collectors.groupingBy(User::getUuid));
        for (TimeAndUserParam item : timeAndUser) {
            for (RelevanceUserParam userParam : item.getUser()) {
                List<User> list = map.get(userParam.getUuid());
                if (CollectionUtils.isEmpty(list)) {
                    return Result.no(Status.EXPECTATION_FAILED, "工号为:" + userParam.getUuid() + "员工不存在");
                }
                userParam.setRealName(list.get(0).getRealName());
            }
        }
        ruleParam.setDuration(null);
        ruleParam.setDistance(null);
        ruleParam.setTimeAndNumber(new ArrayList<>());
        return Result.ok();
    }

    @Override
    public LocationFenceJobVo scanAlarm(LocationFenceJobParam param) {
        RuleParam rule = param.getRule();
        Set<LocationTag> tagSet = param.getTagSet();
        Long mapId = param.getMapId();
        List<LocationFenceAlarm> oldAlarms = param.getOldAlarms();
        Long fenceId = param.getFenceId();
        Long alarmTypeId = param.getAlarmTypeId();
        LocationFenceJobVo jobVo = new LocationFenceJobVo();
        List<TimeAndUserParam> timeAndUserParamList = rule.getTimeAndUser();
        TimeAndUserParam timeAndUserParam = null;
        StringBuilder begin = new StringBuilder();
        StringBuilder end = new StringBuilder();
        //筛选在当前时间生效的配置
        for (TimeAndUserParam item : timeAndUserParamList) {
            for (TimeParam time : item.getTime()) {
                if (TimeUtil.judgeWeek(LocalDateTime.now(), time.getWeek())) {
                    if (TimeUtil.judgeTime(LocalDateTime.now(), time.getBeginTime(), time.getEndTime())) {
                        timeAndUserParam = item;
                        begin.append(time.getWeek()).append(" ").append(time.getBeginTime());
                        end.append(time.getWeek()).append(" ").append(time.getEndTime());
                        break;
                    }
                }
            }
        }
        //如果不在配置的时间内，则将报警记录中的状态刷新为结束，并且需要添加的报警为空
        if (timeAndUserParam == null) {
            jobVo.setModifyAlarm(oldAlarms);
            return jobVo;
        }
        //筛选出在配置时间段内出这个围栏的进出信息
        List<LocationTurnover> turnoverList = locationTurnoverRepository.withinFence(fenceId, false);
        //筛选出在配置时间段内出这个围栏的进出信息
        turnoverList =
            turnoverList
                .stream()
                .filter(e -> TimeUtil.judgeTime(e.getTime(), begin.toString(), end.toString()))
                .sorted(Comparator.comparing(LocationTurnover::getTime).reversed())
                .collect(Collectors.toList());
        //筛选出离开围栏的人员是不允许离开的人员
        Set<String> uuids = timeAndUserParam
            .getUser()
            .stream()
            .map(RelevanceUserParam::getUuid)
            .collect(Collectors.toSet());
        List<LocationFenceAlarm> newAlarms = turnoverList
            .stream()
            .filter(e -> uuids.contains(e.getTag().getUser().getUuid()))
            .map(e -> {
                LocationFence fence = LocationFence.builder().id(fenceId).name(param.getFenceName()).build();
                AlarmType alarmType = AlarmType.builder().id(alarmTypeId).build();
                return LocationFenceAlarm
                    .builder()
                    .fence(fence)
                    .point(e.getTag().getPoint())
                    .areaId(mapId)
                    .user(e.getTag().getUser())
                    .tag(e.getTag())
                    .startTime(LocalDateTime.now())
                    .type(alarmType)
                    .status(LocationFenceAlarm.Status.ING)
                    .build();
            })
            .collect(Collectors.toList());
        //取两个集合没有相交的部分，newAlarms不重合的部分代表需要添加的报警，oldAlarms不重合的部分代表需要结束的报警
        IScanAlarmStrategy.notIntersect(newAlarms, oldAlarms, jobVo);
        return jobVo;
    }
}
