/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.key.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.key.KeyAlarm;
import com.yiring.app.domain.key.KeyAlarmRepository;
import com.yiring.app.domain.key.KeyAlarmRule;
import com.yiring.app.domain.key.KeyAlarmRuleRepository;
import com.yiring.app.excel.key.KeyAlarmExportExcel;
import com.yiring.app.param.key.KeyAlarmFindParam;
import com.yiring.app.service.key.KeyAlarmService;
import com.yiring.app.vo.key.KeyAlarmVo;
import com.yiring.app.vo.user.UserListInfoVo;
import com.yiring.app.vo.user.UserVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.jetbrains.annotations.NotNull;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 按键报警
 *
 * @author LJ-2204
 * @date 2022/5/10
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class KeyAlarmServiceImpl implements KeyAlarmService {

    @Resource
    KeyAlarmRepository keyAlarmRepository;

    @Resource
    KeyAlarmRuleRepository keyAlarmRuleRepository;

    @Override
    public Result<PageVo<KeyAlarmVo>> page(KeyAlarmFindParam param, PageParam pageParam) {
        Specification<KeyAlarm> specification = condition(param);

        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<KeyAlarm> keyAlarms = keyAlarmRepository.findAll(specification, pageable);
        List<KeyAlarmVo> keyAlarmVos = keyAlarms
            .stream()
            .map(keyAlarm -> {
                KeyAlarmVo keyAlarmVo = KeyAlarmVo.transform(keyAlarm);
                User leader = keyAlarm.getLeader();

                if (ObjectUtil.isNotEmpty(leader)) {
                    UserVo userVo = UserVo
                        .builder()
                        .id(leader.getId())
                        .realName(leader.getRealName())
                        .uuid(leader.getUuid())
                        .departmentId(leader.getDepartment().getId())
                        .build();
                    keyAlarmVo.setUser(userVo);
                }
                Optional<KeyAlarmRule> keyAlarmRuleOptional = keyAlarmRuleRepository.findOne(
                    Example.of(KeyAlarmRule.builder().department(leader.getDepartment()).build())
                );

                if (keyAlarmRuleOptional.isPresent()) {
                    KeyAlarmRule keyAlarmRule = keyAlarmRuleOptional.get();
                    Set<User> users = keyAlarmRule.getUsers();
                    List<String> names = users.stream().map(User::getRealName).collect(Collectors.toList());
                    keyAlarmVo.setReceiver(StrUtil.join(",", names));
                    keyAlarmVo.setTypes(keyAlarmRule.getTypes().split(","));
                }

                return keyAlarmVo;
            })
            .collect(Collectors.toList());

        PageVo<KeyAlarmVo> pageVo = PageVo.build(keyAlarmVos, keyAlarms.getTotalElements());

        return Result.ok(pageVo);
    }

    @Override
    public void export(KeyAlarmFindParam param, HttpServletResponse response) {
        Specification<KeyAlarm> specification = condition(param);
        List<KeyAlarm> keyAlarms = keyAlarmRepository.findAll(specification);
        List<KeyAlarmExportExcel> keyAlarmExportExcels = keyAlarms
            .stream()
            .map(keyAlarm -> {
                KeyAlarmExportExcel keyAlarmExportExcel = KeyAlarmExportExcel.transform(keyAlarm);
                User leader = keyAlarm.getLeader();

                if (ObjectUtil.isNotEmpty(leader)) {
                    keyAlarmExportExcel.setRealName(leader.getRealName());
                    keyAlarmExportExcel.setUuid(leader.getUuid());
                }
                Optional<KeyAlarmRule> keyAlarmRuleOptional = keyAlarmRuleRepository.findOne(
                    Example.of(KeyAlarmRule.builder().department(leader.getDepartment()).build())
                );

                if (keyAlarmRuleOptional.isPresent()) {
                    KeyAlarmRule keyAlarmRule = keyAlarmRuleOptional.get();
                    Set<User> users = keyAlarmRule.getUsers();
                    List<String> names = users.stream().map(User::getRealName).collect(Collectors.toList());
                    keyAlarmExportExcel.setReceiver(StrUtil.join(",", names));
                }
                return keyAlarmExportExcel;
            })
            .collect(Collectors.toList());

        try (
            DefaultStreamExcelBuilder<KeyAlarmExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(KeyAlarmExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(keyAlarmExportExcels);

            String fileName = URLEncoder.encode("按键报警信息.xlsx", StandardCharsets.UTF_8);

            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);

            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出按键报警信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<PageVo<KeyAlarmVo>> find(KeyAlarmFindParam param) {
        Specification<KeyAlarm> specification = condition(param);

        List<KeyAlarm> keyAlarms = keyAlarmRepository.findAll(specification);
        List<KeyAlarmVo> keyAlarmVos = keyAlarms
            .stream()
            .map(keyAlarm -> {
                KeyAlarmVo keyAlarmVo = KeyAlarmVo.transform(keyAlarm);
                User leader = keyAlarm.getLeader();

                if (ObjectUtil.isNotEmpty(leader)) {
                    UserVo userVo = UserVo
                        .builder()
                        .id(leader.getId())
                        .realName(leader.getRealName())
                        .uuid(leader.getUuid())
                        .departmentId(leader.getDepartment().getId())
                        .build();
                    keyAlarmVo.setUser(userVo);
                }
                Optional<KeyAlarmRule> keyAlarmRuleOptional = keyAlarmRuleRepository.findOne(
                    Example.of(KeyAlarmRule.builder().department(leader.getDepartment()).build())
                );

                if (keyAlarmRuleOptional.isPresent()) {
                    KeyAlarmRule keyAlarmRule = keyAlarmRuleOptional.get();
                    Set<User> users = keyAlarmRule.getUsers();
                    List<String> names = users.stream().map(User::getRealName).collect(Collectors.toList());
                    keyAlarmVo.setReceiver(StrUtil.join(",", names));
                    keyAlarmVo.setTypes(keyAlarmRule.getTypes().split(","));
                }
                return keyAlarmVo;
            })
            .collect(Collectors.toList());

        PageVo<KeyAlarmVo> pageVo = PageVo.build(keyAlarmVos, keyAlarmVos.size());

        return Result.ok(pageVo);
    }

    @NotNull
    private Specification<KeyAlarm> condition(KeyAlarmFindParam param) {
        return (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (ObjectUtil.isNotEmpty(param.getAreaId())) {
                predicates.add(cb.equal(root.get(KeyAlarm.Fields.areaId), param.getAreaId()));
            }

            if (ObjectUtil.isNotEmpty(param.getName())) {
                predicates.add(
                    cb.like(root.get(KeyAlarm.Fields.leader).get(User.Fields.realName), "%" + param.getName() + "%")
                );
            }

            if (ObjectUtil.isNotEmpty(param.getEnable())) {
                predicates.add(cb.equal(root.get(KeyAlarm.Fields.enable), param.getEnable()));
            }

            if (ObjectUtil.isNotEmpty(param.getCreateTime())) {
                predicates.add(cb.greaterThan(root.get(BasicEntity.Fields.createTime), param.getCreateTime()));
            }

            if (ObjectUtil.isNotEmpty(param.getEndTime())) {
                predicates.add(cb.lessThan(root.get(BasicEntity.Fields.updateTime), param.getEndTime()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };
    }

    @Override
    public Result<PageVo<UserListInfoVo>> listInfo(KeyAlarmFindParam param) {
        Specification<KeyAlarm> specification = condition(param);

        List<KeyAlarm> keyAlarms = keyAlarmRepository.findAll(specification);
        List<User> users = keyAlarms.stream().map(KeyAlarm::getLeader).collect(Collectors.toList());

        ArrayList<UserListInfoVo> userListInfoVos = ListUtil.toList();

        // 员工、访客
        Map<User.Type, List<User>> mapTypes = users.stream().collect(Collectors.groupingBy(User::getType));

        // 员工
        List<User> userList = mapTypes.get(User.Type.EMPLOYEES);

        // 普通人员
        AtomicInteger oPer = new AtomicInteger(0);
        // 内部特殊工作人员
        AtomicInteger intSpPer = new AtomicInteger(0);
        // 外部特殊工作人员
        AtomicInteger extSpPer = new AtomicInteger(0);
        // 巡检人员
        AtomicInteger inspectionPer = new AtomicInteger(0);

        userList.forEach(user -> {
            // 特殊人员处理
            if (user.getIsSpecial()) {
                if (StrUtil.isNotEmpty(user.getUnit())) {
                    // 外部特殊人员+
                    extSpPer.addAndGet(1);
                } else {
                    intSpPer.addAndGet(1);
                }
                return;
            }

            if (ObjectUtil.equal(User.Status.INSPECTION, user.getStatus())) {
                inspectionPer.addAndGet(1);
                return;
            }
            oPer.addAndGet(1);
        });

        // 普通人员+
        userListInfoVos.add(UserListInfoVo.builder().type("普通人员").numOfPeo(oPer.get()).build());
        // 内部特殊工作人员+
        userListInfoVos.add(UserListInfoVo.builder().type("内部特殊工作人员").numOfPeo(intSpPer.get()).build());
        // 外部特殊工作人员+
        userListInfoVos.add(UserListInfoVo.builder().type("外部特殊工作人员").numOfPeo(extSpPer.get()).build());
        // 巡检人员+
        userListInfoVos.add(UserListInfoVo.builder().type("巡检人员").numOfPeo(inspectionPer.get()).build());
        // 访客+
        userListInfoVos.add(
            UserListInfoVo.builder().type("访客").numOfPeo(mapTypes.get(User.Type.GUEST).size()).build()
        );

        PageVo<UserListInfoVo> pageVo = PageVo.build(userListInfoVos, users.size());
        return Result.ok(pageVo);
    }
}
