/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.user;

import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.auth.domain.permission.Permission;
import com.yiring.auth.domain.role.Role;
import com.yiring.auth.domain.role.RoleRepository;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.auth.param.IdsParam;
import com.yiring.auth.param.user.*;
import com.yiring.auth.service.user.UserService;
import com.yiring.auth.util.Auths;
import com.yiring.auth.util.Permissions;
import com.yiring.auth.vo.permission.MenuVo;
import com.yiring.auth.vo.user.UserInfoVo;
import com.yiring.auth.vo.user.UserVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 系统用户控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/1/24 14:13
 */

@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@ApiSupport(order = -95)
@Api(tags = "用户管理", description = "User")
@RestController
@RequestMapping("/user/")
public class UserController {

    @Resource
    Auths auths;

    @Resource
    UserRepository userRepository;

    @Resource
    RoleRepository roleRepository;

    @Resource
    UserService userService;

    @ApiOperation(value = "获取登录用户信息")
    @GetMapping("getUserInfo")
    public Result<UserInfoVo> getUserInfo() {
        User user = auths.getLoginUser();
        UserInfoVo userInfoVo = UserInfoVo
            .builder()
            .userId(user.getId())
            .uuid(user.getUuid())
            .mobile(user.getMobile())
            .deptName(user.getDepartment().getName())
            .username(user.getUsername())
            .realName(user.getRealName())
            .avatar(user.getAvatar())
            .desc(user.getIntroduction())
            .roles(Permissions.toRoleVos(user.getRoles()))
            .build();
        return Result.ok(userInfoVo);
    }

    @ApiOperation(value = "获取用户菜单")
    @GetMapping("getMenuList")
    public Result<ArrayList<MenuVo>> getMenuList() {
        User user = auths.getLoginUser();
        List<Permission> permissions = Permissions
            .toPermissions(user.getRoles())
            .stream()
            .filter(permission -> !Permission.Type.BUTTON.equals(permission.getType()))
            .collect(Collectors.toList());
        List<MenuVo> vos = Permissions.toMenuTreeVo(permissions);
        return Result.ok((ArrayList<MenuVo>) vos);
    }

    @ApiOperation(value = "获取用户权限")
    @GetMapping("getPermCode")
    public Result<ArrayList<String>> getPermCode() {
        User user = auths.getLoginUser();
        List<Permission> permissions = Permissions.toPermissions(user.getRoles());
        List<String> codes = permissions.stream().map(Permission::getUid).collect(Collectors.toList());
        return Result.ok((ArrayList<String>) codes);
    }

    @ApiOperation(value = "分配角色")
    @PostMapping("/manage/assign")
    public Result<String> assign(@Valid IdParam idParam, @Valid IdsParam idsParam) {
        Optional<User> optional = userRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        // 查询权限集合
        Set<Long> ids = idsParam.toIds();
        Set<Role> roles = roleRepository
            .findAll()
            .stream()
            .filter(role -> ids.contains(role.getId()))
            .collect(Collectors.toSet());

        User entity = optional.get();
        entity.setRoles(roles);
        userRepository.saveAndFlush(entity);
        return Result.ok();
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/manage/page")
    public Result<PageVo<UserVo>> page(@Valid UserFindParam param, @Valid PageParam pageParam) {
        return userService.page(param, pageParam);
        /*  Page<User> page = userRepository.findAll(PageParam.toPageable(pageParam));
        List<UserVo> data = page
            .get()
            .map(role -> {
                UserVo vo = new UserVo();
                BeanUtils.copyProperties(role, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<UserVo> vo = PageVo.build(data, page.getTotalElements());
        return Result.ok(vo);*/
    }

    @ApiOperation("添加用户")
    @PostMapping("add")
    public Result<String> add(@Valid UserSaveParam param) {
        return userService.add(param);
    }

    @ApiOperation("重置密码")
    @PutMapping("reset")
    public Result<String> reset(@Valid IdParam idParam) {
        return userService.reset(idParam);
    }

    @ApiOperation("变更状态")
    @PutMapping("upStatus")
    public Result<String> upStatus(@Valid IdParam idParam) {
        return userService.upStatus(idParam);
    }

    @ApiOperation(value = "导出", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("export")
    public void export(@Valid UserExportParam param, HttpServletResponse response) {
        userService.export(param, response);
    }

    @ApiOperation("修改密码")
    @PutMapping("modifyPwd")
    public Result<String> modifyPwd(@Valid UserPwdParam param, @Valid IdParam idParam) {
        return userService.modifyPwd(param, idParam);
    }

    @ApiOperation("修改手机号")
    @PutMapping("modifyPhone")
    public Result<String> modifyPhone(@Valid UserPhoneParam param, @Valid IdParam idParam) {
        return userService.modifyPhone(param, idParam);
    }

    @ApiOperation("通过手机号码修改密码")
    @PutMapping("modifyPwdByP")
    public Result<String> modifyPwdByPhone(@Valid UserPwdParam userPwdParam, @Valid UserPhoneParam userPhoneParam) {
        return userService.modifyPwdByPhone(userPwdParam, userPhoneParam);
    }

    @ApiOperation("发送验证码")
    @PutMapping("sendCode")
    public Result<String> sendCode(@Valid UserPhoneParam param) {
        return userService.sendCode(param);
    }
}
